/*   rgpio.h
 *
 *   A driver for General-Purpose I/O devices.
 *
 *   (C) Copyright 2002 Fred Gleason <fredg@wava.com>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public
 *   License along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#ifndef RGPIO_H
#define RGPIO_H

#include <qobject.h>
#include <qtimer.h>
#include <qsignalmapper.h>

#define GPIO_CLOCK_INTERVAL 100
#define GPIO_MAX_LINES 24

/**
 * @short A driver for Genral Purpose I/O Devices
 * @author Fred Gleason <fredg@wava.com>
 * 
 * This class implements a driver for a GPIO device.  With the exception
 * of the ASI driver, it utilizes direct I/O port calls, thus, root 
 * permissions are needed when executing the constructor, following which 
 * root permissions may be safely dropped.
 **/
class RGpio : public QObject
{
 Q_OBJECT
 public:
 /**
  * GPIO Board Types
  **/
 enum Board {Unknown=0,PDIS08=1,PDIS16=2,DIO24=3,GAME=4,ASI=5};

 /**
  * Board Mode
  **/
 enum Mode {Auto=0,Input=1,Output=2};

 /**
  * Instantiates the widget.
  * @param board The type of GPIO board.
  * @param port The base port address of the GPIO board.  Not needed for ASI.
  * @param mode The mode of the board.  Needed only for the DIO24.
  **/
  RGpio(Board board,unsigned short port=0,Mode mode=RGpio::Auto,
	QObject *parent=0,const char *name=0);

 /**
  * Indicate whether the driver has successfully gotten permissions to
  * access the I/O ports.  Root permissions are needed for this to
  * succeed!
  * NOTE:  root permissions are not required to access the ASI driver.
  * Returns: True = Success, false = Failure.
  **/
  bool authorized() const;

 /**
  * Returns the number of available input (GPI) lines available.
  **/
  int inputs() const;

 /**
  * Returns the number of available output (GPO) lines available.
  **/
  int outputs() const;

 /**
  * Returns the index of the first input channel
  **/
  int inputIndex() const;

 /**
  * Set the Input Index value
  * @param index = The value to set the index to
  **/
  void setInputIndex(int index);

 /**
  * Set the Input Index value
  * @param index = The value to set the index to
  **/
  void setOutputIndex(int index);

 /**
  * Returns the index of the first output channel
  **/
  int outputIndex() const;

 /**
  * Returns the current input mask
  **/
  unsigned inputMask();

 /**
  * Returns the current state of the indicated GPI line
  * @param line Line number
  **/
  bool inputState(int line);

 /**
  * Returns the current output mask
  **/
  unsigned outputMask() const;


 signals:
  /** 
   * Emitted upon a change to an input (GPI) line
   * @param pin The number of the GPI line
   * @param state True = Line set, false = line clear
   **/
  void inputChanged(int line,bool state);

 public slots:
 /**
  * Set (turn on) a given output (GPO) line.
  * @param line The line to set.
  * @param interval The length of time to hold the new state (in mS).  
  * 0 = hold new state indefinetly.
  **/
  void gpoSet(int line,unsigned interval=0);

 /**
  * Reset (turn off) a given output (GPO) line.
  * @param line The line to clear.
  * @param interval The length of time to hold the new state (in mS).  
  * 0 = hold new state indefinetly.
  **/
  void gpoReset(int line,unsigned interval=0);

 private slots:
  void timerData();
  void revertData(int id);

 private:
  void SetReversion(int line,unsigned interval);
  void WriteRegister();
  void ReadRegister();
  RGpio::Board gpio_board;
  RGpio::Mode gpio_mode;
  QTimer *gpio_timer;
  QSignalMapper *gpio_revert_mapper;
  QTimer *gpio_revert_timer[GPIO_MAX_LINES];
  bool gpio_revert_state[GPIO_MAX_LINES];
  int gpio_input_index;
  int gpio_output_index;
  unsigned short gpio_iobase;
  unsigned gpio_in_register;
  unsigned gpio_out_register;
  unsigned input_size;
  unsigned output_size;
  bool gpio_auth;
};


#endif
