#ifndef RINPUTMODULE_H
#define RINPUTMODULE_H

#include <qwidget.h>
#include <qstring.h>
#include <qcolor.h>
#include <qpainter.h>
#include <qsize.h>
#include <qslider.h>
#include <qsizepolicy.h>
#include <qpushbutton.h>
#include <qbuttongroup.h>
#include <rnamespace.h>
#include <rslider.h>

/**
 * @short Implements an input module for an audio mixing desk.
 * @author Fred Gleason <fredg@wava.com>
 * 
 * This class implements an input module for an audio mixing desk.  It
 * includes a source selector, buss assignment buttons, a fader control
 * and on/off buttons.
 **/
class RInputModule : public QWidget
{
 Q_OBJECT
 public:
 /**
  * Instantiates the widget.
  * @param channel Sets the channel number of the widget.
  * @param width Sets the width of the widget, in pixels.
  **/
  RInputModule(int channel,QWidget *parent=0,const char *name=0,int width=78);

 /**
  * Display/hide the source selector.
  * @param state true=Show the selector, false = hide the selector.
  **/
  void showSelector(bool state);

  /**
   * Set the source selector labels.
   * @param label_a The text of the label to associate with Input A
   * @param label_b The text of the label to associate with Input B
   **/
  void labelSelector(QString label_a,QString label_b);

  /**
   * Set the source selector colors.
   * @param color_a The color to associate with Input A
   * @param color_b The color to associate with Input B
   **/
  void colorSelector(QColor color_a,QColor color_b);

  /**
   * Set the display state of a buss assignment button.
   * @param button The button number (0-5).
   * @param state true = Show the button, false = Don't show the button.
   **/
  void showButton(int button,bool state);

  /**
   * Set the label of a buss assignment button.
   * @param button The button number (0-5).
   * @param label The label to associate with the button.
   **/
  void labelButton(int button,QString label);

  /**
   * Set the color of a buss assignment button.
   * @param button The button number (0-5).
   * @param color The color to associate with the button.
   **/
  void colorButton(int button,QColor color);

  /**
   * Set the display state of the audio fader.
   * @param state true = Show the fader, false = Don't display the fader.
   **/
  void showFader(bool state);

  /**
   * Set the color of the audio fader.
   * @param color The color to associate with the fader.
   **/
  void colorFader(QColor color);

  /**
   * Set the range of the audio fader.
   * @param min The minimum value of the fader.
   * @param max The maximum value of the fader.
   **/
  void setRangeFader(int min,int max);

  /**
   * Set the gradation interval of the fader tick marks.
   * @param interval The interval.
   **/
  void setTickFader(int interval);

  /**
   * Set the display state of the on/off buttons.
   * @param state true = Show on/off buttons, false = Don't show on/off buttons
   **/
  void showOnOff(bool state);

  /**
   * Set the label text to be associated with the on button.
   * @param on_label The text to be associated with the on button.
   **/
  void labelOn(QString on_label);

  /**
   * Set the colors associated with on on/off buttons.
   * @param on_color The color associated with the on button.
   * @param off_color The color associated with the off button.
   **/
  void colorOnOff(QColor on_color,QColor off_color);

  /**
   * Returns a recommended size for the widget.
   **/
  QSize sizeHint() const;

  /**
   * Returns the layout behavior of the widget.
   **/
  QSizePolicy sizePolicy() const;

 public slots:

 /**
  * Selects the source on the audio source selector.
  * @param channel Channel number of the widget.
  * @param source The audio source, 0 = Source A, 1 = Source B.
  **/
  void pushSelector(int channel,int source);
 
 /**
  * Set the on/off state of buss button 0
  * @param channel Channel number of the widget.
  * @param state true = On, false = Off.
  **/    
  void pushButton0(int channel,bool state);
 
 /**
  * Set the on/off state of buss button 1
  * @param channel Channel number of the widget.
  * @param state true = On, false = Off.
  **/    
  void pushButton1(int channel,bool state);
 
 /**
  * Set the on/off state of buss button 2
  * @param channel Channel number of the widget.
  * @param state true = On, false = Off.
  **/    
  void pushButton2(int channel,bool state);
 
 /**
  * Set the on/off state of buss button 3
  * @param channel Channel number of the widget.
  * @param state true = On, false = Off.
  **/    
  void pushButton3(int channel,bool state);
 
 /**
  * Set the on/off state of buss button 4
  * @param channel Channel number of the widget.
  * @param state true = On, false = Off.
  **/    
  void pushButton4(int channel,bool state);
 
 /**
  * Set the on/off state of buss button 5
  * @param channel Channel number of the widget.
  * @param state true = On, false = Off.
  **/    
  void pushButton5(int channel,bool state);

 /**
  * Set the fader to a given level.
  * @param channel Channel number of the widget.
  * @param gain The setting to put the fader to.
  **/
  void setFader(int channel,int gain);

 /**
  * The the state of the on/off buttons.
  * @param channel Channel number of the widget.
  * @param state true = On, false = Off.
  **/
  void setOnOff(int channel,bool state);

 signals:
 /**
  * Indicates a change in the selector state.
  * @param channel Channel number of the widget.
  * @param source The audio source, 0 = Source A, 1 = Source B.
  **/
  void selectorPushed(int channel,int source);

 /**
  * Indicates a change in state of buss selector button 0.
  * @param channel Channel number of the widget.
  * @param state true = On, false = Off.
  **/
  void button0Pushed(int channel,bool state);

 /**
  * Indicates a change in state of buss selector button 1.
  * @param channel Channel number of the widget.
  * @param state true = On, false = Off.
  **/
  void button1Pushed(int channel,bool state);

 /**
  * Indicates a change in state of buss selector button 2.
  * @param channel Channel number of the widget.
  * @param state true = On, false = Off.
  **/
  void button2Pushed(int channel,bool state);

 /**
  * Indicates a change in state of buss selector button 3.
  * @param channel Channel number of the widget.
  * @param state true = On, false = Off.
  **/
  void button3Pushed(int channel,bool state);

 /**
  * Indicates a change in state of buss selector button 4.
  * @param channel Channel number of the widget.
  * @param state true = On, false = Off.
  **/
  void button4Pushed(int channel,bool state);

 /**
  * Indicates a change in state of buss selector button 5.
  * @param channel Channel number of the widget.
  * @param state true = On, false = Off.
  **/
  void button5Pushed(int channel,bool state);

 /**
  * Indicates a change in fader setting.
  * @param channel Channel number of the widget.
  * @param gain The setting to put the fader to.
  **/
  void faderSet(int channel,int gain);

 /**
  * Indicates a change in on/off button status.
  * @param channel Channel number of the widget.
  * @param state true = On, false = Off.
  **/
  void isOnOff(int channel,bool state);

 private slots:
  void selectorClicked();
  void button0Clicked();
  void button1Clicked();
  void button2Clicked();
  void button3Clicked();
  void button4Clicked();
  void button5Clicked();
  void faderValue(int);
  void onOffClicked(int);

 protected:
  void paintEvent(QPaintEvent *);

 private:
  int c;
  int w,ht;
  void runLayout();
  QPushButton *selector;
  QString selector_label_a,selector_label_b;
  QColor selector_color_a,selector_color_b;
  int selector_status;
  QPushButton *buss[6];
  RSlider *fader;
  int fader_min,fader_max;
  QPushButton *on_button,*off_button;
  QButtonGroup *on_off_button;
};


#endif
