/*   rplaystream.h
 *
 *   A class for playing Microsoft WAV files.
 *
 *   (C) Copyright 2002 Fred Gleason <fredg@wava.com>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public
 *   License along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#ifndef RPLAYSTREAM_H
#define RPLAYSTREAM_H

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <qobject.h>
#include <qwidget.h>
#include <qstring.h>
#include <qdatetime.h>
#include <qtimer.h>
#include <qdatetime.h>
#include <rnamespace.h>
#include <rwavefile.h>
#include <rsoundcard.h>

#ifdef HPI
#include <hpi/hpi.h>
#define AUDIO_SIZE 32768
#define FRAGMENT_TIME 50
#endif

#ifdef ALSA
#include <alsa/asoundlib.h>
#define AUDIO_SIZE 32768
#define ALSA_PERIOD_SIZE 8192
#define ALSA_DEV "es1968"
#define FRAGMENT_TIME 50
#define DEFAULT_ALSA_PLAY_DEVICE "plughw"
#endif

/**
 * @short An object for playing Microsoft WAV files
 * @author Fred Gleason <fredg@wava.com>
 *
 * RPlayStream implements a virtual device for playing Microsoft WAV
 * files.  It supports AudioScience HPI audio hardware as well as devices
 * using the Advanced Linux Sound Architecture (ALSA).
 * It is capable of playing PCM8, PCM16 and (if the underlying hardware
 * supports it) MPEG-1 formatted files.
 **/
class RPlayStream : public QObject,public RWaveFile
{
  Q_OBJECT

  public:
  /**
   * Defines the current state of the machine.
   **/
  enum State {Stopped=0,Playing=1,Paused=2};

  /**
   * Instantiates the virtual device.
   * @param hpi_handle Pointer to HPI subsystem handle.
   * @param parent Parent object.
   * @param name Name of object.
   **/
   RPlayStream(QWidget *parent=0,const char *name=0);

  /**
   * Destroys the virtual device.
   **/
   ~RPlayStream();

  /**
   * Loads a wave file into the virtual device.  Returns
   * true of successful, otherwise false.
   * @param filename Pathname of the WAV file to be loaded.  
   **/
   bool loadWave(QString filename);

  /**
   * Unloads the loaded WAV file from the vitual device.
   **/
   void unloadWave();

  /**
   * Returns the adapter card number the virtual device is set to use.
   **/
   int getCard() const;

  /**
   * Returns the adapter stream number the virtual device is set to use.
   **/
   int getStream() const;

  /**
   * Sets the playout speed of the machine.
   * @param speed The playout speed, in tenths of percent.
   * @param pitch True = allow pitch to shift in proportion to
   * speed change, false = don't.
   * @param rate True = Allow output sample rate to vary in
   * proportion to speed change, false = don't.
   * 
   * Returns: True if successful, false if parameter combination not
   * supported by the underlying hardware.
   *
   * NOTE: Not all adapters support all combinations of parameters.  In
   * particular, ASI4000 series adapters cannot vary playout
   * speed without varying both pitch and sample rate in proportion, which
   * can cause problems when using IEC958-type output devices.  
   **/
   bool setSpeed(int speed,bool pitch=false,bool rate=false);

   /**
    * Returns the current ALSA device.  This has no meaning for HPI adapters.
    **/
   QString device() const;

   /**
    * Set the ALSA PCM device to use for playout.  This has no effect on
    * ASI adapters.
    * @param device The ALSA PCM device to use for playout.
    **/
   void setDevice(QString device);

  /**
   * Returns the current device state.
   **/
   RPlayStream::State getState() const;

  /**
   * Returns true if a playback meter is available, otherwise false.
   **/
   bool haveMeter() const;

  /**
   * Returns true if a volume control for the corresponding output port
   * exists, otherwise false.
   * @param port Output port number.
   **/
   bool haveOutputVolume(int port);

  /**
   * Returns the current channel volume setting for the corresponding 
   * output port.
   * @param port Output port number.
   **/
   int getOutputVolume(int port);

  signals:
  /**
   * Emitted whenever the stop status changes
   * @param state true=stream is stopped,  false =  stream is playing or 
   * paused.
   **/
   void isStopped(bool state);

  /**
   * Emitted whenever the virtual machine enters 'play' mode.
   **/
   void played();

  /**
   * Emitted whenever the virtual machine enters 'pause' mode.
   **/
   void paused();

  /**
   * Emitted whenever the virtual machine enters 'stop' mde.
   **/
   void stopped();

  /**
   * Emitted periodically whenever the machine is in 'play' mode.  The
   * can be used to implement a playback progress indicator.
   * @param samples Number of samples played since beginning of file.
   **/
   void position(int samples);

  /**
   * Emitted periodically whenever the machine is in 'play' mode, if
   * the underlying hardware supports a output stream meter.
   * @param level The current left level of the playing stream, in 1/100 of a
   * dB below digital zero.
   **/
   void leftMeter(int level);

  /**
   * Emitted periodically whenever the machine is in 'play' mode, if
   * the underlying hardware supports a output stream meter.
   * @param level The current right level of the playing stream, in 1/100 of a
   * dB below digital zero.
   **/
   void rightMeter(int level);

  /**
   * Emitted whenever the state of the machine changes.
   * @param card The card number.
   * @param stream The stream number.
   * @param state The new machine state.
   **/
   void stateChanged(int card,int stream,RPlayStream::State state);

  public slots:
  /**
   * Set the adapter card number to use.
   * @param card The adapter card number.
   **/
   void setCard(int card);

  /**
   * Set the adapter stream number to use.
   * @param stream The adapter stream number.
   **/
   void setStream(int stream);

  /**
   * Put the virtual machine into 'play' mode.
   **/
   bool play();

  /**
   * Put the virtual machine into 'pause' mode.
   **/
   void pause();

  /**
   * Put the virtual machine into 'stop' mode.
   **/
   void stop();

  /**
   * Set the playback position in the WAV file.  Any subsequent playout
   * will start from this location.
   * @param samples Location in sample in the WAV file.
   **/
   void setPosition(int samples);

  /**
   * Set the volume control for the corresponding output port.
   * @param port Output port number.
   * @param level Level in 1/100 of a dB below digital zero.
   **/
   void setOutputVolume(int port,int level);

  /**
   * Set the fade type to be used by 'fadeOutputVolume()'.
   * @param type The type of fade to be used.
   **/
   void setFadeType(RSoundCard::FadeProfile type);

  /**
   * Start an auto-fade on the selected output port.
   * @param port The output port.
   * @param level The level desired at the end of the fade, in 1/100 of a dB
   * below digital zero.
   * @param length The time length of the fade, in milliseconds.
   **/
   void fadeOutputVolume(int port,int level,int length);

  /**
   * Set the length of playout.  0 = play until told to stop or the end
   * of the file is reached.  (NOTE: the machine will be placed into
   * 'pause' mode at the expiration of this time, NOT stop!)
   * @param length Length of playout in milliseconds.
   **/
   void setPlayLength(int length);


#ifdef HPI
  private slots:
   void leftMeterSample(int card,int stream,int level);
   void rightMeterSample(int card,int stream,int level);

  private:
   RSoundCard *sound_card;
   RPlayStream::State stream_state;
   void drainStream();
   QString wave_name;
   QTimer *clock;
   int card_number;
   int stream_number;
   bool playing;
   bool is_paused;
   bool repositioned;
   bool stopping;
   unsigned audio_ptr;
   unsigned char abuf[AUDIO_SIZE];
   unsigned left_to_write;
   unsigned samples_skipped;
   int play_length;
   QTimer *play_timer;
   QTime start_time;
   int play_speed;
   bool pitch_can_vary;
   bool rate_can_vary;
   HPI_HOSTREAM hpi_stream;
   HW16 state;
   HW32 buffer_size;
   HW32 data_to_play;
   HW32 samples_played;
   HW32 reserved;
   int fragment_size;
   int fragment_time;
   HW8 *pdata;
   HPI_FORMAT format;
   HPI_DATA hpi_data;

   private slots:
    void tickClock();
#endif  // HPI

#ifdef ALSA
  private slots:
   void leftMeterSample(int card,int stream,int level);
   void rightMeterSample(int card,int stream,int level);
   void tickClock();

  private:
   RSoundCard *sound_card;
   RPlayStream::State stream_state;
   void drainStream();
   ssize_t ReadWave(void *buf,size_t count);
   QString wave_name;
   QTimer *clock;
   int card_number;
   int stream_number;
   bool playing;
   bool is_paused;
   bool repositioned;
   bool stopping;
//   unsigned audio_ptr;
//   unsigned left_to_write;
//   unsigned samples_skipped;
   int play_length;
   QTimer *play_timer;
   int play_speed;
   bool pitch_can_vary;
   bool rate_can_vary;
//   int fragment_size;
//   int fragment_time;
   char sound_buffer[AUDIO_SIZE];
   int current_position;

   snd_pcm_t *pcm_handle;
   snd_pcm_stream_t stream;
   snd_pcm_hw_params_t *hwparams;
   QString alsa_device;
#endif  // ALSA
};


#endif  // RPLAYSTREAM_H

