/*   rpushbutton.h
 *
 *   An flashing button widget.
 *
 *   (C) Copyright 2002-2003 Fred Gleason <fredg@salemradiolabs.com>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public
 *   License along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#ifndef RPUSHBUTTON_H
#define RPUSHBUTTON_H

#include <qwidget.h>
#include <qpushbutton.h>
#include <qpixmap.h>
#include <qcolor.h>
#include <rnamespace.h>

/*
 * Widget Defaults
 */
#define RPUSHBUTTON_DEFAULT_FLASH_PERIOD 300


/**
 * @short A flashing push button
 * @author Fred Gleason <fredg@wava.com
 * 
 * This class implements a standard QPushButton, with the addition of 
 * the ability to flash.
 **/
class RPushButton : public QPushButton
{
  Q_OBJECT

  public:
  /**
   * Instantiates the widget.
   * @param parent The parent widget.
   * @param name The name of the widget.
   **/
   RPushButton(QWidget *parent,const char *name=0);

  /**
   * Instantiates the widget.
   * @param parent The parent widget.
   * @param name The name of the widget.
   * @param text The text of the widget.
   **/
   RPushButton(const QString &text,QWidget *parent,const char *name=0);

  /**
   * Instantiates the widget.
   * @param parent The parent widget.
   * @param name The name of the widget.
   * @param text The text of the widget.
   * @param icon The icons of the widget.
   **/
   RPushButton(const QIconSet &icon,const QString &text,
	       QWidget *parent,const char *name=0);

   /**
    * Get the flash color of the button
    **/
   QColor flashColor() const;

   /**
    * Set the flash color of the button
    * @param color the color to use for the flash color.
    **/
   void setFlashColor(QColor color);

   /**
    * Get the flash period of the button
    **/
   int flashPeriod() const;

   /**
    * Set the flash period.
    * @param period = Flash period in mS
    **/
   void setFlashPeriod(int period);

  public slots:
  /**
   * Get flashing state
   **/
   bool flashingEnabled() const;

  /**
   * Set the flashing state
   **/
  void setFlashingEnabled(bool state);

  /**
   * A reimplementation from QWidget for internal reasons, the API is 
   * unchanged.
   **/
  void setPalette(const QPalette &);

  signals:
  /**
   * Emitted when the center mouse button is clicked on the widget.
   **/
   void centerClicked();

  /**
   * Emitted when the center mouse button is pressed on the widget.
   **/
   void centerPressed();

  /**
   * Emitted when the center mouse button is released on the widget.
   **/
   void centerReleased();

  /**
   * Emitted when the right mouse button is clicked on the widget.
   **/
   void rightClicked();

  /**
   * Emitted when the right mouse button is pressed on the widget.
   **/
   void rightPressed();

  /**
   * Emitted when the right mouse button is released on the widget.
   **/
   void rightReleased();

  protected:
   void mousePressEvent(QMouseEvent *e);
   void mouseReleaseEvent(QMouseEvent *e);

  private slots:
   void flashClock();

  private:
   void flashOn();
   void flashOff();
   void Init();
   bool flash_state;
   int flash_period;
   bool flashing_enabled;
   QColor flash_color;
   QPalette flash_palette;
   QPalette off_palette;
   QTimer *flash_timer;
};



#endif
