/*   rrecordstream.h
 *
 *   A class for recording Microsoft WAV files.
 *
 *   (C) Copyright 2002 Fred Gleason <fredg@wava.com>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public
 *   License along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#ifndef RRECORDSTREAM_H
#define RRECORDSTREAM_H

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <qobject.h>
#include <qwidget.h>
#include <qstring.h>
#include <qdatetime.h>
#include <qtimer.h>
#include <rnamespace.h>
#include <rwavefile.h>
#include <rsoundcard.h>

#define DEBUG_VAR "_RRECORDSTREAM"
#define XRUN_VAR "_RSOUND_XRUN"

#ifdef HPI
#include <hpi/hpi.h>
#define AUDIO_SIZE 32768
#define RRECORDSTREAM_CLOCK_INTERVAL 100
#endif

#ifdef ALSA
#include <alsa/asoundlib.h>
#define ALSA_MAX_BUFFER_SIZE 131072
#define ALSA_PERIODS 2
#endif

/**
 * @short An object for recording Microsoft WAV files
 * @author Fred Gleason <fredg@wava.com>
 *
 * RRecordStream implements a virtual device for recording Microsoft WAV
 * files.  It supports AudioScience HPI audio hardware.
 * It is capable of recording PCM8, PCM16 and (if the underlying hardware
 * supports it) MPEG-1 formatted files.
 *
 * If the environmental variable '_RRECORDSTREAM' is defined in the 
 * environment, RRecordStream print debugging information to stdout. 
 **/
class RRecordStream : public QObject,public RWaveFile
{
  Q_OBJECT

  public:
  /**
   * Possible machine states.
   **/
  enum RecordState {Recording=0,RecordReady=1,Paused=2,Stopped=3,
		    RecordStarted=4};

  /**
   * Instantiates the virtual device.
   * @param hpi_handle Pointer to HPI subsystem handle.
   * @param parent Parent object.
   * @param name Name of object.
   **/
   RRecordStream(QWidget *parent=0,const char *name=0);

  /**
   * Destroy an RRecordStream object.
   **/
   ~RRecordStream();

  /**
   * Returns the adapter card number the virtual device is set to use.
   **/
   int getCard() const;

  /**
   * Returns the adapter stream number the virtual device is set to use.
   **/
   int getStream() const;

  /**
   * Returns true if a record meter is available, otherwise false.
   **/
   bool haveMeter() const;

  /**
   * Returns true if a record vox device is available, otherwise false.
   **/
   bool haveInputVOX() const;

  /**
   * Returns true if a volume control for the selected input stream
   * exists, otherwise false.
   **/
   bool haveInputVolume();

  /**
   * Returns the current volume setting for the selected 
   * input stream.
   **/
   int getInputVolume();

  /**
   * Returns the current machine state.
   **/
   RRecordStream::RecordState getState();


  signals:
  /**
   * Emitted whenever the stop status changes
   * @param state true=stream is stopped,  false =  stream is recording or 
   * paused.
   **/
   void isStopped(bool state);

  /**
   * Emitted whenever the virtual machine enters 'RecordReady' mode.
   **/
   void ready();

  /**
   * Emitted whenever the virtual machine enters 'Record' mode.
   **/
   void recording();

  /**
   * Emitted whenever the virtual machine actually begins recording.  In some
   * cases (e.g. a VOX event), recording may start some time after the
   * machine is placed into 'Record' mode.
   **/
   void recordStart();

  /**
   * Emitted whenever the virtual machine enters 'Pause' mode.
   **/
   void paused();

  /**
   * Emitted whenever the virtual machine enters 'Stop' mde.
   **/
   void stopped();

  /**
   * Emitted periodically whenever the machine is in 'record' mode.  This
   * can be used to implement a record timer.
   * @param samples Number of samples recorded since beginning of file.
   **/
   void position(int samples);

  /**
   * Emitted periodically whenever the machine is in 'record' mode, if
   * the underlying hardware supports an input stream meter.
   * @param level The current left level of the recording stream, in 1/100 of a
   * dB below digital zero.
   **/
   void leftMeter(int level);

  /**
   * Emitted periodically whenever the machine is in 'record' mode, if
   * the underlying hardware supports an input stream meter.
   * @param level The current right level of the recording stream, in 1/100 of
   * a dB below digital zero.
   **/
   void rightMeter(int level);

  /**
   * Emitted whenever the state of the machine changes.
   * @param card The card number.
   * @param stream The stream number.
   * @param state The new machine state.
   **/
   void stateChanged(int card,int stream,RRecordStream::RecordState state);

  public slots:
  /**
   * Set the adapter card number to use.
   * @param card The adapter card number.
   **/
   void setCard(int card);

  /**
   * Set the adapter stream number to use.
   * @param stream The adapter stream number.
   **/
   void setStream(int stream);

   /**
    * Returns the current ALSA device.  This has no meaning for HPI adapters.
    **/
   QString device() const;

   /**
    * Set the ALSA PCM device to use for playout.  This has no effect on
    * ASI adapters.
    * @param device The ALSA PCM device to use for playout.
    **/
   void setDevice(QString device);

  /**
   * Put the virtual machine into 'RecordReady' mode.  This creates the
   * WAV file, activates the input metering and does everything else
   * needed to record except actually write sample data into the WAV file.
   **/
   bool recordReady();

  /**
   * Put the virtual machine into 'Record' mode.  If the machine is not
   * already in RecordReady mode, it will first be placed into it.
   **/
   void record();

  /**
   * Put the virtual machine into 'Pause' mode.
   **/
   void pause();

  /**
   * Put the virtual machine into 'Stop' mode.
   **/
   void stop();

  /**
   * Set the volume control for the selected input stream.
   * @param level Level in 1/100 of a dB below digital zero.
   **/
   void setInputVolume(int level);

  /**
   * Set the VOX threhold level.
   * @param gain Threhold level in 1/100 of a dB below digital zero.
   **/
   void setInputVOX(int gain);

  /**
   * Set the length of recording.  0 = record until told to stop.
   * (NOTE:  the machine will be placed into 'pause' mode at the expiration
   * of this time, NOT stop!)
   * @param length Length of recording in milliseconds.
   **/
   void setRecordLength(int length);

#ifdef HPI
  private slots:
   void leftMeterSample(int card,int stream,int level);
   void rightMeterSample(int card,int stream,int level);

  private:
   RSoundCard *sound_card;
   bool debug;
   bool xrun;
   void drainStream();
   QTimer *clock;
   int card_number;
   int stream_number;
   bool is_recording;
   bool is_ready;
   bool is_paused;
   bool stopping;
   bool record_started;
   QTimer *length_timer;
   int record_length;
   unsigned audio_ptr;
   unsigned char abuf[AUDIO_SIZE];
   unsigned left_to_write;
   HPI_HISTREAM hpi_stream;
   HW16 state;
   HW32 buffer_size;
   HW32 data_recorded;
   HW32 samples_recorded;
   HW32 reserved;
   int fragment_size;
   int fragment_time;
   HW8 *pdata;
   HPI_FORMAT format;
   HPI_DATA hpi_data;

   private slots:
    void tickClock();
#endif  // HPI

#ifdef ALSA
  private slots:
   void leftMeterSample(int card,int stream,int level);
   void rightMeterSample(int card,int stream,int level);
   void tickClock();

  private:
   RSoundCard *sound_card;
   bool debug;
   bool xrun;
   QTimer *clock;
   int card_number;
   int stream_number;
   bool is_recording;
   bool is_ready;
   bool is_paused;
   bool stopping;
   bool record_started;
   QTimer *length_timer;
   int record_length;
   unsigned audio_ptr;
   unsigned left_to_write;
   int fragment_size;
   int fragment_time;
   int current_position;

   snd_pcm_t *pcm_handle;
   snd_pcm_hw_params_t *hwparams;
   QString alsa_device;
   int buffer_frames;
   int period_frames;
   char *sound_buffer;
   int clock_interval;
#endif  // ALSA
};


#endif  // RRECORDSTREAM_H

