#ifndef RREV12ENGINE_H
#define RREV12ENGINE_H

#include <qobject.h>
#include <qsocket.h>
#include <qstring.h>
#include <rnamespace.h>

/**
 * @short Controls a Colorado Digital Revolution-12 Digital Audio Engine
 * @author Fred Gleason <fredg@wava.com>
 *
 * This class implements an object for controlling a Colorado Digital
 * Revolution 12 Digital Audio Engine.  Connection to the engine is by means
 * a TCP/IP connection utilizing the 'revd' control daemon.
 **/

class RRev12Engine : public QObject
{
  Q_OBJECT
  public:
    RRev12Engine();
  /**
   * Opens a TCP/IP connection to the 'revd' process controlling the engine.
   * Upon the successful establishment of a connection, the 'connected()'
   * signal will be emitted.  Any errors in the process will emit an 'error()'.
   * @param hostname The hostname or IP address of the 'revd' process.
   * @param port The TCP port number to connect to.
   * @param password The password for authentication with the 'revd' daemon
   **/     
  void openConnection(const QString &hostname,Q_UINT16 port,
		     const QString password=NULL);
  /**
   * Enable/disable level metering
   * @param state true = enable meter updates, false = disable updates
   **/
  void enableLevelMetering(bool state);

  signals:
  /**
   * Connection established.  Indicates that a valid TCP connection to the
   * engine has been established.
   **/
  void connected();
  /**
   * Socket error.  Indicates an error is the TCP socket connection with
   * the engine.
   **/
  void error(int errornum);
  /**
   * Indicates the state of the PGM buss assignment for the indicated
   * channel.
   * @param channel Input channel
   * @param state true = PGM buss assigned, false = PGM buss not assigned
   **/
  void isPgmSet(int channel,bool state);
  /**
   * Indicates successful completion of the closeCompletion() command.
   **/
  void connectionClosed();
  /**
   * Indicates the state of the AUD buss assignment for the indicated
   * channel.
   * @param channel Input channel
   * @param state true = AUD buss assigned, false = AUD buss not assigned
   **/
  void isAudSet(int channel,bool state);
  /**
   * Indicates the state of the UTL buss assignment for the indicated
   * channel.
   * @param channel Input channel
   * @param state true = UTL buss assigned, false = UTL buss not assigned
   **/
  void isUtlSet(int channel,bool state);
  /**
   * Indicates the state of the M1 buss assignment for the indicated
   * channel.
   * @param channel Input channel
   * @param state true = M1 buss assigned, false = M1 buss not assigned
   **/
  void isM1Set(int channel,bool state);
  /**
   * Indicates the state of the M2 buss assignment for the indicated
   * channel.
   * @param channel Input channel
   * @param state true = M2 buss assigned, false = M2 buss not assigned
   **/
  void isM2Set(int channel,bool state);
  /**
   * Indicates the state of the CUE buss assignment for the indicated
   * channel.
   * @param channel Input channel
   * @param state true =  CUE buss assigned, false = CUE buss not assigned
   **/
  void isCueSet(int channel,bool state);
  /**
   * Indicates the audio source for the indicated channel.  0 = Input A,
   * 1 = Input B
   * @param channel Input Channel
   * @param source 0 = Input A, 1 = Input B
   **/
  void channelSource(int channel,int source);
  /**
   * Indicates the ON/OFF state of the channel.
   * @param channel Input Channel
   * @param state true = Channel is ON, false = Channel is OFF
   **/
  void isChannelEnabled(int channel,bool state);
  /**
   * Indicates the gain setting of the channel.
   * @param channel Input Channel
   * @param gain Gain in dB X 1000
   **/
  void getChannelVolume(int channel,int gain);
  /**
   * Indicates channel mode
   * @param channel input Channel 
   * @param mode 0 = Line, 1 = Mic
   **/
  void getChannelMode(int channel,int mode);
  /**
   * Indicates Cue Volume
   * @ param gain Gain in dB X 1000
   **/
  void getCueVolume(int gain);
  /**
   * Indicates AutoCue Mode
   * @param state true = AutoCue enabled, false = AutoCue disabled 
   **/
  void getAutoCueMode(bool state);
  /**
   * Indicates Headset Volume
   * @param gain Gain in dB X 1000
   **/
  void getHeadsetVolume(int gain);
  /**
   * Indicates Control Room Monitor Source
   * @param source 0 = PGM, 1 = AUD, 2 = UTL, 3 = Ext 1, 4 = Ext 2
   **/
  void getCrSource(int source);
  /**
   * Indicates Control Room Monitor Volume
   * @param gain Gain in dB X 1000
   **/
  void getCrVolume(int gain);
  /**
   * Indicates Control Room Monitor Mute State
   * @param state true = Monitor muted, false = Monitor not muted
   **/
  void getCrMuteState(bool state);
  /**
   * Indicates Studio Monitor Source
   * @param source 0 = PGM, 1 = AUD, 2 = UTL
   **/
  void getStudioSource(int source);
  /**
   * Indicates Studio Monitor Volume
   * @param gain Gain in dB X 1000
   **/
  void getStudioVolume(int gain);
  /**
   * Indicates Studio Monitor Mute State
   * @param state true = Monitor muted, false = Monitor not muted
   **/
  void getStudioMuteState(bool state);
  /**
   * Indicates Talkback Group Membership
   * @param channel Input Channel (1-5)
   * @param state true=member of tb group, false = not member of tb group
   **/
  void getTbGroup(int channel,bool state);
  /**
   * Indicates Talkback Tally Status
   * @param state true = Tally active, false = Tally not active
   **/
  void getTbTally(bool state);
  /**
   * Indicates Studio Talkback State
   * @param state true = TB activated, false = TB not activated
   **/
  void getStudioTbActive(bool state);
  /**
   * Indicates Control Room Talkback State
   * @param state true = TB activated, false = TB not activated
   **/
  void getCrTbActive(bool state);

  /**
   * PGM Buss Left Audio Meter Level
   * @param level Left meter level in dB X 100
   **/
  void getPgmMeterLeft(int level);

  /**
   * PGM Buss Right Audio Meter Level
   * @param level Right meter level in dB X 100
   **/
  void getPgmMeterRight(int level);

  /**
   * AUD Buss Left Audio Meter Level
   * @param level Left meter level in dB X 100
   **/
  void getAudMeterLeft(int level);

  /**
   * AUD Buss Right Audio Meter Level
   * @param level Right meter level in dB X 100
   **/
  void getAudMeterRight(int level);

  public slots:
  /**
   * Closes the TCP/IP connection to the 'revd' daemon.  The connectionClosed()
   * signal is emitted to indicate successful completion of the close.
   **/
  void closeConnection();
  /**
   * Causes the object to emit signals to indicate the full state of the
   * engine.  Useful for initializing things when joining a session in
   * progress.
   **/
  void sync();
  /**
   * Set PGM Buss Assignment
   * @param channel Input Channel
   * @param state true = ON, false = OFF
   **/
    void setPgm(int channel,bool state);
  /**
   * Set AUD Buss Assignment
   * @param channel Input Channel
   * @param state true = ON, false = OFF
   **/
  void setAud(int channel,bool state);
  /**
   * Set UTL Buss Assignment
   * @param channel Input Channel
   * @param state true = ON, false = OFF
   **/
  void setUtl(int channel,bool state);
  /**
   * Set Mix Minus 1 Buss Assignment
   * @param channel Input Channel
   * @param state true = ON, false = OFF
   **/
  void setM1(int channel,bool state);
  /**
   * Set Mix Minus 2 Buss Assignment
   * @param channel Input Channel
   * @param state true = ON, false = OFF
   **/
  void setM2(int channel,bool state);
  /**
   * Set Cue Buss Assignment
   * @param channel Input Channel
   * @param state true = ON, false = OFF
   **/
  void setCue(int channel,bool state);
  /**
   * Set Channel Audio Source
   * @param channel Input Channel
   * @param source Audio Source, 0 = Channel A, 1 = Channel B 
   **/
  void setChannelSource(int channel,int source);
  /**
   * Set Channel ON/OFF State
   * @param channel Input Channel
   * @param state true = ON, false = OFF
   **/
  void setChannelEnabled(int channel,bool state);
  /**
   * Set Channel Volume
   * @param channel Input Channel
   * @param gain Gain in dB X 1000
   **/
  void setChannelVolume(int channel,int gain);
  /**
   * Set Channel Mode, 0 = Line, 1 = Mic
   * @param channel Input Channel
   * @param mode Channel Mode, 0 = Line, 1 = Mic
   * NOTE: This setting does NOT affect the input sensitivity of the channel
   * -- a jumper on the Rev12 motherboard is used for that.  Rather, this
   * controls how the channel is routed internally (MONO vs. STEREO).
   **/
  void setChannelMode(int channel,int mode);
  /**
   * Set Cue Volume
   * @param gain Gain in dB X 1000
   **/
  void setCueVolume(int gain);
  /**
   * Set AutoCue Mode
   * @param state true = AutoCue ON, false = AutoCue OFF
   **/
  void setAutoCueMode(bool state);
  /**
   * Set Headset Volume
   * @param gain Gain in dB X 1000
   **/
  void setHeadsetVolume(int gain);
  /**
   * Set Control Room Monitor Source
   * @param source 0 = PGM, 1 = AUD, 2 = UTL, 3 = Ext 1, 4 = Ext2
   **/
  void setCrSource(int source);
  /**
   * Set Control Room Monitor Volume
   * @param gain Gain in dB X 1000
   **/
  void setCrVolume(int gain);
  /**
   * Set the Control Room Monitor Mute State
   * param state true = Monitor Muted, false = Monitor not muted
   **/
  void setCrMuteState(bool state);
  /**
   * Set Studio Monitor Audio Source
   * @param source 0 = PGM,1 = AUD, 2 = UTL
   **/
  void setStudioSource(int source);
  /**
   * Set Studio Monitor Volume
   * @param gain Gain in dB X 1000
   **/
  void setStudioVolume(int gain);
  /**
   * Set Studio Monitor Mute State
   * @param state true = Monitor Muted, false = Monitor not muted
   **/
  void setStudioMuteState(bool state);
  /**
   * Set Talkback Group Membership
   * @param state true = Belongs to TB Group, false = Does not belong to TB
   **/
  void setTbGroup(int channel,bool state);
  /**
   * Set Talkback Tally Output
   * @param state true = Tally active, false = Tally not active
   **/
  void setTbTally(bool state);
  /**
   * Set Studio Talkback State
   * @param state true = TB active, false = TB not active
   **/
  void setStudioTbActive(bool state);
  /**
   * Set Control Room Talkback State
   * @param state true = TB active, false = TB not active
   **/
  void setCrTbActive(bool state);

  private slots:
  void socketConnected();
  void socketDisconnected();
  void readSocket();
  
  private:
  void toEngine(const char *);
  void parseSocket(const char *);
  int faderGainToRatio(int);
  int faderRatioToGain(int);
  int monitorGainToRatio(int);
  int monitorRatioToGain(int);
  int cueGainToRatio(int);
  int cueRatioToGain(int);
  int meterRatioToGain(int);
  QSocket socket;
  char pw[256];
  bool connectionState;
  char accum[256];
};

#endif
