/*   rslider.h
 *
 *   An audio- and touchscreen-friendly slider widget.
 *
 *   (C) Copyright 2002 Fred Gleason <fredg@wava.com>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public
 *   License along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#ifndef RSLIDER_H
#define RSLIDER_H

#include <qwidget.h>
#include <qslider.h>
#include <qrangecontrol.h>
#include <qcolor.h>
#include <qpalette.h>
#include <qsize.h>
#include <qpixmap.h>
#include <rnamespace.h>


/**
 * @short An audio- and touchscreen-friendly slider control.
 * @author Fred Gleason <fredg@wava.com>
 *
 * RSlider is a slider widget that is closely modeled on the API used
 * by QSlider, with three primary additions:  the ability to set the size
 * of the knob independently of the size of the widget, the ability to
 * set the color of the knob independently of the color of the widget,
 * and the ability to set the direction of the range control --e.g. does
 * moving the slider UP make the corresponding value increase or decrease?
 **/
class RSlider : public QWidget,public QRangeControl
{
  Q_OBJECT

  public:
  /**
   * Create a RSlider with the following parameters:
   * @param parent The parent widget
   * @param name The name of the widget
   **/
   RSlider(QWidget *parent,const char *name);

  /**
   * Create a RSlider with the following parameters:
   * @param orient The orientation of the slider.
   * @param parent The parent widget
   * @param name The name of the widget
   **/
   RSlider(Radio::Orientation orient,QWidget *parent,const char *name);

  /**
   * Create a RSlider with the following parameters:
   * @param minValue The minimum value of the range.
   * @param maxValue The maximum value of the range.
   * @param pageStep The increment by whcih the value will change upon
   * receipt of a pageStep command.
   * @param value The initial value of the slider.
   * @param orient The orientation of the slider.
   * @param parent The parent widget
   * @param name The name of the widget
   **/
   RSlider(int minValue,int maxValue,int pageStep,int value,
	   Radio::Orientation orient,QWidget *parent,const char *name);

  /**
   * Returns the orientation of the slider.
   **/
   Radio::Orientation orientation() const;

  /**
   * Set the orientation of the slider.  The orientation determines
   * whether the slider will be drawn vertically or horizontally, as 
   * well as which direction of motion will result in an increasing
   * value.
   * @param orient The orieintation to use.
   **/ 
   void setOrientation(Radio::Orientation orient);

  /**
   * This works just the same as with the QSlider.
   **/
   void setTracking(bool enable);

  /**
   * This works just the same as with the QSlider.
   **/
   bool tracking() const;

  /**
   * This works just the same as with the QSlider.
   **/
   void setTickInterval(int i);

  /**
   * This works just the same as with the QSlider.
   **/
   void setTickmarks(QSlider::TickSetting s);

  /**
   * This works just the same as with the QSlider.
   **/
   void setMinValue(int min_value);

  /**
   * This works just the same as with the QSlider.
   **/
   void setMaxValue(int max_value);

  /**
   * This works just the same as with the QSlider.
   **/
   void setRange(int min_value,int max_value);

  /**
   * This works just the same as with the QSlider.
   **/
   void setLineStep(int step);

  /**
   * This works just the same as with the QSlider.
   **/
   void setPageStep(int step);

  /**
   * Set the color of the knob.  The 'Dark', 'Shadow' and 'Light' roles
   * in the color group will be used to render the 3D effects.
   * @param group Color group to use for the knob.
   **/
   void setKnobColor(QColorGroup group);

  /**
   * This method is a reimplementation of the one above, and differs only
   * in what arguments it accepts.
   * @param color Color to use for the knob.
   **/
   void setKnobColor(QColor color);

  /**
   * Set the size of the knob.
   * @param size Size of knob.
   **/
   void setKnobSize(QSize size);

  /**
   * This method is a reimplementation of the one above, and differs only
   * in what arguments it accepts.
   * @param w The width of the knob.
   * @param h The height of the knob.
   **/
   void setKnobSize(int w,int h);

  signals:

  /**
   * This works just the same as with the QSlider.
   **/
   void valueChanged(int value);

  /**
   * This works just the same as with the QSlider.
   **/
   void sliderPressed();

  /**
   * This works just the same as with the QSlider.
   **/
   void sliderMoved(int value);

  /**
   * This works just the same as with the QSlider.
   **/
   void sliderReleased();

  public slots:
  /**
   * Reimplemented for internal reasons, the API is unaffected.
   **/
   void setGeometry(int x,int y,int w, int h);

  /**
   * Reimplemented for internal reasons, the API is unaffected.
   **/
   void setGeometry(QRect &rect);

  /**
   * This works just the same as with the QSlider.
   **/
   void setValue(int setting);

  /**
   * This works just the same as with the QSlider.
   **/
   void addStep();

  /**
   * This works just the same as with the QSlider.
   **/
   void subtractStep();

  private slots:

  private:
   void paintEvent(QPaintEvent *);
   void mouseMoveEvent(QMouseEvent *);
   void mousePressEvent(QMouseEvent *);
   void mouseReleaseEvent(QMouseEvent *);
   void init();
   void calcKnob();
   void calcKnob(int x,int y,int w,int h);
   Radio::Orientation rslider_orient;
   QColorGroup knob_color;
   QRect curr_knob;
   QRect prev_knob;
   QRect page_up;
   QRect page_down;
   QSize knob_size;
   QPixmap knob_map;
   int base_x;
   int base_y;
   bool rslider_moving;
   int page_step;
   int line_step;
   bool tracking_enabled;
   bool deferred_change;
   int tick_interval;
   QSlider::TickSetting tick_setting;
};



#endif
