/*   rstereometer.h
 *
 *   A Stereo Audio Meter Widget
 *
 *   (C) Copyright 2002 Fred Gleason <fredg@wava.com>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public
 *   License along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#ifndef RSTEREOMETER_H
#define RSTEREOMETER_H

#include <qwidget.h>
#include <qcolor.h>
#include <qpainter.h>
#include <qsize.h>
#include <qsizepolicy.h>
#include <qstring.h>
#include <rnamespace.h>
#include <rsegmeter.h>

#define CLIP_LIGHT_COLOR red


/**
 * @short A calibrated stereo audio level meter
 * This class implements a calibrated, stereo audio level meter, similar to
 * that found on many pieces of audio gear.  A calibrated scale and meter
 * label is included.
 * @author Fred Gleason <fredg@wava.com>
 **/
class RStereoMeter : public QWidget
{
 Q_OBJECT
 public:
 /**
  * Instantiates the widget.
  **/
  RStereoMeter(QWidget *parent=0,const char *name=0);

 /**
  * Returns a recommended size for the widget.
  **/
  QSize sizeHint() const;

  /**
   * Returns the sizing policy for the widget.
   **/
  QSizePolicy sizePolicy() const;

  /**
   * Sets the reference level for the widget --i.e. the point which will
   * display as '0' dB on the meter.  If set to '0', the reference point
   * will be -16 dBFS.
   * @param level Reference level
   **/
  void setReference(int level);

  /**
   * Sets the level at which the CLIP light will come on, referenced to the 
   * level specified in 'setReference()' above.
   * @param level The clip light level.
   **/
  void setClipLight(int level);

  /**
   * Sets the color of the "dark" (i.e. unlit) segments in the first level
   * of the meter.
   * @param color The color to assign to the dark segments
   **/
  void setDarkLowColor(QColor color);

  /**
   * Sets the color of the "dark" (i.e. unlit) segments in the second level
   * of the meter.
   * @param color The color to assign to the dark segments
   **/
  void setDarkHighColor(QColor color);

  /**
   * Sets the color of the "dark" (i.e. unlit) segments in the third level
   * of the meter.
   * @param color The color to assign to the dark segments
   **/
  void setDarkClipColor(QColor color);

  /**
   * Sets the color of the first level range of segments in the meter.
   * @param color The color to assign to the first level range of segments
   **/
  void setLowColor(QColor color);

  /** Sets the color of the second level range of segments in the meter.
   * @param color The color to assign to the second level range of segments
   **/
  void setHighColor(QColor color);

  /**
   * Sets the color of the third level range of segments in the meter.
   * @param color The color to assign to the third level range of segments
   **/
  void setClipColor(QColor color);

  /**
   * Sets the threshold level for the transition point between the first
   * and second level range of segments
   * @param level The threshold point between the first and second level
   * range of segment
   **/
  void setHighThreshold(int level);

  /**
   * Sets the threshold level for the transition point between the second
   * and third level range of segments
   * @param level The threshold point between the second and third level
   * range of segment
   **/
  void setClipThreshold(int level);

  /**
   * Sets the width of the segments in the meters
   * @param size Width of segments in the meters
   **/
  void setSegmentSize(int size);

  /**
   * Sets the width of the gap between segments in the meters
   * @param size Width of gap between segments in the meters
   **/
  void setSegmentGap(int gap);

  /**
   * Set the labels text attached to the widget
   * @param label Text of label
   **/
  void setLabel(QString label);

  /**
   * Get the current meter mode
   **/
  RSegMeter::Mode mode() const;

  /**
   * Set the meter mode.
   * @param mode = The mode to set
   **/
  void setMode(RSegMeter::Mode mode);

 public slots:
  /**
   * Set the level of the left solid bar display
   * @param level Level to set
   **/
  void setLeftSolidBar(int level);

  /**
   * Set the level of the right solid bar display
   * @param level Level to set
   **/
  void setRightSolidBar(int level);

  /**
   * Set the level of the left floating bar display
   * @param level Level to set
   **/
  void setLeftFloatingBar(int level);

  /**
   * Set the level of the right floating bar display
   * @param level Level to set
   **/
  void setRightFloatingBar(int level);

  /**
   * Set the level of the left peak bar display
   * @param level Level to set
   **/
  void setLeftPeakBar(int level);

  /**
   * Set the level of the right peak bar display
   * @param level Level to set
   **/
  void setRightPeakBar(int level);

  /**
   * Resets (turns off) the CLIP light.
   **/
  void resetClipLight();

 signals:
  /**
   * Emitted when the CLIP light is activated
   **/
  void clip();

 protected:
  void paintEvent(QPaintEvent *);

 private:
  RSegMeter *left_meter,*right_meter;
  int ref_level;
  int clip_light_level;
  bool clip_light_on;
  int label_x;
  QString meter_label;
};


#endif
