/*   rtelos100.h
 *
 *   A Qt driver for the Telos 100 series of broadcast telephone interfaces.
 *
 *   (C) Copyright 2002 Fred Gleason <fredg@wava.com>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public
 *   License along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#ifndef RTELOS100_H
#define RTELOS100_H

#include <qobject.h>
#include <qstring.h>
#include <qtimer.h>
#include <rnamespace.h>
#include <rttydevice.h>

/*
 * Global Defines
 */
#define RTELOS100_WATCHDOG_INTERVAL 10000
#define RTELOS100_MAX_LINES 10
#define RTELOS100_MAX_BANKS 2
#define RTELOS100_TICK_INTERVAL 100
#define RTELOS100_DEBUG_VAR "_RTELOS100"


/**
 * @short Controls a Telos 100 Series Digital Telephone Interface
 * @author Fred Gleason <fredg@wava.com>
 *
 * This class implements an object for controlling a Telos100 Series
 * Digital Telephone Interface.  It works by establishing a serial
 * connection to the unit.
 **/

class RTelos100 : public QObject
{
  Q_OBJECT
  public:
   enum Command {Next=15,Hold=24,Drop=25,SelectLine=26,DialKey=14,BusyAll=51,
                 DelayDump=52,Record=54,RequestLineState=5,
                 RadioScreenedHold=62,ReturnStatus=129,
		 LineState=134,BusyAllState=145,RecordingState=146,
		 DelayDumpState=147};

  /**
   * Create an RTelos100 object
   *
   * @param watchdog True = Try to autodetect failures and reconnect, 
   * false = don't.
   **/
  RTelos100(int id,bool watchdog=false,QObject *parent=0,const char *name=0);

  /**
   * Opens a serial connection to the Telos 100.
   * @param port The tty port to use.
   * @param speed The speed to use for the connection, in bits/sec.
   * Returns: True = connection successful, false = not successful
   **/     
  bool openConnection(const QString port,unsigned speed=1200);

  /**
   * This send a command to the Telos 100.
   * @param cmd The command code
   * @param line The line number
   * @param group The group number
   * @param alt The alternate field
   * @param console The console number
   * @param bank The bank number
   **/
   void sendCommand(RTelos100::Command cmd,int line,int alt,int bank);

  signals:
  /**
   * This is emitted once for each status change received from the Telos 100.
   * @param reply The command code
   * @param line The line number
   * @param group The group number
   * @param alt The alternate field
   * @param console The console number
   * @param bank The bank number
   **/
   void receivedReply(int id,RTelos100::Command reply,
		      int line,int alt,int bank);

  public slots:
  /**
   * Closes the serial connection to the daemon.
   * Returns true = close successful, false = unsuccessful
   **/
   void closeConnection(int id);

  protected:

  private slots:
   void ReceiveTTY();
   void WatchdogData();

  private:
   void SendState(int line);
   void DispatchStatus(int cmd,int line);
   RTTYDevice *telos_tty;
   int telos_id;
   bool telos_watchdog;
   QTimer *telos_watchdog_timer;
   char telos_state[RTELOS100_MAX_LINES];
   bool telos_lock[RTELOS100_MAX_LINES];
   bool telos_record;
   QTimer *telos_clock;
   bool telos_debug;
};

#endif
