/*   rtelostwox12.h
 *
 *   A Qt driver for the Telos TWOx12 Digital Telephone Interace
 *
 *   (C) Copyright 2002 Fred Gleason <fredg@wava.com>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public
 *   License along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#ifndef RTELOSTWOX12_H
#define RTELOSTWOX12_H

#include <qobject.h>
#include <qsocket.h>
#include <qstring.h>
#include <rnamespace.h>

/*
 * Global Defines
 */
#define RTWOX12_DEFAULT_TCP_PORT 5001
#define RTWOX12_WATCHDOG_INTERVAL 10000
#define RTWOX12_MAX_LINES 12
#define RTWOX12_MAX_BANKS 2
#define RTWOX12_MAX_CONSOLES 16
#define RTWOX12_DEBUG_VAR "_RTELOSTWOX12"

/**
 * @short Controls a Telos Systems TWOx12 Digital Telephone Interface
 * @author Fred Gleason <fredg@wava.com>
 *
 * This class implements an object for controlling a Telos Systems TWOx12
 * Digital Telephone Interface.  It works by establishing a TCP/IP
 * connection to the unit.
 **/

class RTelosTwoX12 : public QObject
{
  Q_OBJECT
  public:
   enum Command {StudioLogin=2,StudioLogout=3,SelectDesktopDirector=20,
                 Next=15,Hold=24,Drop=25,SelectLine=26,DialKey=14,BusyAll=51,
                 DelayDump=52,Record=54,ChangeDirectorMode=18,
                 PasswordRequiredQuery=21,SendPassword=1,
                 RequestLineState=5,SendCallerData=6,MessageIn=7,
                 RequestLineConfiguration=10,RequestDirectorNames=17,
                 RequestServerTime=22,RequestDirectorMode=27,
		 RadioRequestProtocol=63,RadioScreenedHold=62,
		 RadioFlash=61,RadioMute=60,RadioConf=59,RadioVip=58,
                 RadioScreen=57,
		 PasswordResponse=126,ReturnStudioNames=128,ReturnStatus=129,
		 ReturnCallerData=133,LineState=134,MessageOut=135,
		 ReturnDirectorName=137,DirectorPhoneExists=138,
		 PasswordRequiredResponse=139,ReturnServerTime=142,
		 BusyAllState=145,RecordingState=146,
		 DelayDumpState=147,ReturnClientMode=149,
		 RadioFlashState=190,RadioMuteState=189,
		 RadioConfState=188,RadioVipState=187,RadioScreenState=186,
                 RadioReturnProtocol=191};
   enum Protocol {Unknown=0,TelosTwoX12=1,Telos100=2,TS612=3};

  /**
   * Create an RTelosTwoX12 object
   *
   * @param watchdog True = Try to autodetect failures and reconnect, 
   * false = don't.
   **/
  RTelosTwoX12(int id,bool watchdog=false,QObject *parent=0,const char *name=0);

  /**
   * Opens a TCP/IP connection to the TWOx12.
   * @param hostname The hostname or IP address of the TWOx12.
   * @param port The TCP port number to connect to.
   * @param password The password for authentication with the TWOx12.
   * Returns: True = connection successful, false = not successful
   **/     
  void openConnection(const QString &hostname,const QString password="",
		      Q_UINT16 port=RTWOX12_DEFAULT_TCP_PORT);

  /**
   * Sets the internal socket device to use the specified socket and 
   * the state to 'connected'.  This allows us to use the class to
   * implement a Telos protocol server.
   *
   * @param socket The fd of the socket to use.  It should already be 
   * connected.
   **/
  void setSocket(int socket);

  /**
   * This send a command to the TWOx12.
   * @param reply The reply code
   * @param line The line number
   * @param group The group number
   * @param alt The alternate field
   * @param console The console number
   * @param bank The bank number
   * @param size The size of the buffer
   * @param buffer Pointer to the buffer data
   **/
   void sendCommand(RTelosTwoX12::Command cmd,int line,int group,int alt,
		    int console,int bank,unsigned size,const char *buffer);

   /**
    * The gets a string consisting of the local system's IP address and source
    * port being used for the TCP connection.  This can be used to uniquely
    * identify the connection (useful when passing caller and message data
    * to/from the TWOx12).
    **/
   QString connectionID() const;

  signals:
  /**
   * This is emitted when the connection has attempted to log into the
   * TWOx12.
   * @param The result of the login attempt, true = successful, false = not
  **/
  void authenticated(int id,bool result);

  /**
   * Socket error.  Indicates an error in the TCP socket connection with
   * the engine.
   **/
  void error(int id,int errornum);

  /**
   * Indicates successful completion of the closeConnection() command.
   **/
  void connectionClosed(int id);

  /**
   * This is emitted once for each packet received from the TWOx12.
   * @param reply The reply code
   * @param line The line number
   * @param group The group number
   * @param alt The alternate field
   * @param console The console number
   * @param bank The bank number
   * @param size The size of the buffer
   * @param buffer Pointer to the buffer data
   **/
   void receivedReply(int id,RTelosTwoX12::Command reply,int line,int group,
		     int alt,int console,int bank,unsigned size,
		     char *buffer);

  public slots:
  /**
   * Closes the TCP/IP connection to the daemon.  The connectionClosed()
   * signal is emitted to indicate successful completion of the close.
   **/
   void closeConnection(int id);

  protected:

  private slots:
   void connectedData();
   void connectionClosedData();
   void readyReadData();
   void watchdogData();
  
  private:
   void ReceivePacket();
   QString DumpCommand(int cmd);
   bool CheckReply(RTelosTwoX12::Command reply,int line,int group,int alt,
		   int console,int bank,int size,char *buf);
   int telos_id;
   QString telos_hostname;
   QString telos_password;
   Q_UINT16 telos_port;
   QSocket *telos_socket;
   RTelosTwoX12::Command current_reply;
   bool password_tried;
   QTimer *telos_watchdog_timer;
   bool telos_watchdog;
   bool telos_watchdog_awake;
   bool telos_debug;
};

#endif
