/*   rttydevice.h
 *
 *   A Qt driver for tty ports on Linux.
 *
 *   (C) Copyright 2002 Fred Gleason <fredg@wava.com>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public
 *   License along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#ifndef RTTYDEVICE_H
#define RTTYDEVICE_H

#include <termios.h>
#include <unistd.h>
#include <qiodevice.h>
#include <qstring.h>
#include <rnamespace.h>


/**
 * @short A QIODevice-compatible class for Linux ttys.
 * @author Fred Gleason <fredg@wava.com>
 *
 * This class implements an QIODevice-compatible class for dealing 
 * with tty (i.e. COM port) devices on Linux.
 **/

class RTTYDevice : public QIODevice
{
  public:
   enum Parity {None=0,Even=1,Odd=2};

  /**
   * Create an RTTYDevice object
   **/
  RTTYDevice();

  /**
   * Destroy an RTTYDevice object
   **/
  ~RTTYDevice();

  /**
   * This method is reimplemented from the QIODevice class, the API is
   * unaffected.
   **/
  bool open(int mode);

  /**
   * This method is reimplemented from the QIODevice class, the API is
   * unaffected.
   **/
  void close();

  /**
   * This method is reimplemented from the QIODevice class, the API is
   * unaffected.
   **/
  void flush();

  /**
   * This method is reimplemented from the QIODevice class, the API is
   * unaffected.
   **/
  Q_LONG readBlock(char *data,Q_ULONG maxlen);

  /**
   * This method is reimplemented from the QIODevice class, the API is
   * unaffected.
   **/
  Q_LONG writeBlock(const char *data,Q_ULONG len);

  /**
   * This method is reimplemented from the QIODevice class, the API is
   * unaffected.
   **/
  int getch();

  /**
   * This method is reimplemented from the QIODevice class, the API is
   * unaffected.
   **/
  int putch(int ch);

  /**
   * This method is reimplemented from the QIODevice class, the API is
   * unaffected.
   **/
  int ungetch(int ch);


  /**
   * This method is reimplemented from the QIODevice class, the API is
   * unaffected.
   **/
  QIODevice::Offset size() const;

  /**
   * This method is reimplemented from the QIODevice class, the API is
   * unaffected.
   **/
  int flags() const;

  /**
   * This method is reimplemented from the QIODevice class, the API is
   * unaffected.
   **/
  int mode() const;

  /**
   * This method is reimplemented from the QIODevice class, the API is
   * unaffected.
   **/
  int state() const;

  /**
   * This method is reimplemented from the QIODevice class, the API is
   * unaffected.
   **/
  bool isDirectAccess() const;

  /**
   * This method is reimplemented from the QIODevice class, the API is
   * unaffected.
   **/
  bool isSequentialAccess() const;

  /**
   * This method is reimplemented from the QIODevice class, the API is
   * unaffected.
   **/
  bool isCombinedAccess() const;

  /**
   * This method is reimplemented from the QIODevice class, the API is
   * unaffected.
   **/
  bool isBuffered() const;

  /**
   * This method is reimplemented from the QIODevice class, the API is
   * unaffected.
   **/
  bool isRaw() const;

  /**
   * This method is reimplemented from the QIODevice class, the API is
   * unaffected.
   **/
  bool isSynchronous() const;

  /**
   * This method is reimplemented from the QIODevice class, the API is
   * unaffected.
   **/
  bool isAsynchronous() const;

  /**
   * This method is reimplemented from the QIODevice class, the API is
   * unaffected.
   **/
  bool isTranslated() const;

  /**
   * This method is reimplemented from the QIODevice class, the API is
   * unaffected.
   **/
  bool isReadable() const;

  /**
   * This method is reimplemented from the QIODevice class, the API is
   * unaffected.
   **/
  bool isWritable() const;

  /**
   * This method is reimplemented from the QIODevice class, the API is
   * unaffected.
   **/
  bool isReadWrite() const;

  /**
   * This method is reimplemented from the QIODevice class, the API is
   * unaffected.
   **/
  bool isInactive() const;

  /**
   * This method is reimplemented from the QIODevice class, the API is
   * unaffected.
   **/
  bool isOpen() const;

  /**
   * This method is reimplemented from the QIODevice class, the API is
   * unaffected.
   **/
  int status() const;

  /**
   * This method is reimplemented from the QIODevice class, the API is
   * unaffected.
   **/
  void resetStatus();

  /**
   * Set the name of the tty device to use.
   * @param name Name of tty device to use.
   **/
  void setName(QString name);

  /**
   * Returns the speed of the tty device, in bits per second.
   **/
  int speed() const;

  /**
   * Set the speed of the tty device.
   * @param speed Speed of the tty device, in bits per second.
   **/
  void setSpeed(int speed);

  /**
   * Returns the word length of the tty device.
   **/
  int wordLength() const;

  /**
   * Set the word length of the tty device.
   * @param length Word length in bits.  Legal values are 5, 6, 7 or 8.
   **/
  void setWordLength(int length);

  /**
   * Returns the parity coding of the tty device.
   **/
  RTTYDevice::Parity parity() const;

  /**
   * Set the parity coding of the tty device.
   * @param parity Parity coding.
   **/
  void setParity(Parity);

 private:
  void Init();
  speed_t tty_speed;
  tcflag_t tty_length;
  Parity tty_parity;
  QString tty_name;
  int tty_fd;
  bool tty_open;
  int tty_flags;
  int tty_mode;
  int tty_status;
};


#endif
